# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/lingo.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

import sys

# Maps names of lingos to Interpreters.
# FIXME: the lingo registry should probably be turned into a class
_lingos = {}

def register(name, interpreter):
    """Registers a lingo with the given name and interpreter.  Raises a KeyError if a lingo with
    name is already registered."""
    if name in _lingos:
        raise KeyError('A lingo with name {0} already exists'.format(name))

    _lingos[name] = interpreter

def get(name):
    """Return the interpreter for the lingo of the given name.  If no such lingo exists, return
    None."""
    return _lingos.get(name)

def iter_lingos():
    return _lingos.iteritems()

# The base class for all Interpreters.
class Interpreter(object):
    # FIXME: Once multiple servers are supported, servers should default to the current list of
    # servers and the servers argument here should be removed.
    def __init__(self, console, stdin = sys.stdin, stdout = sys.stdout):
        """Create a new Interpreter object.  stdin will be used as the standard input within the
        lingo and may be either a file or a callable.  If stdin is a callable, each invocation
        should take no arguments and return the next line of input.  Newlines should be retained.
        stdout should be either a file or a callable.  If stdout is a callable it should take one
        string argument.  servers is a list of servers to execute commands on."""
        self.console = console

        if isinstance(stdin, file):
            self.stdin = stdin.readline
        else:
            self.stdin = stdin

        if isinstance(stdout, file):
            self.stdout = stdout.write
        else:
            self.stdout = stdout

    def input_complete(self, command):
        """Indicates if command is a complete command for the given lingo by returning the
        command if it is complete and None otherwise."""
        return command
    
    def read(self, command):
        """Convert the string command to the format used to represent code.  This could be an
        abstract syntax tree or even plain text.  The result must be an object that can be passed to
        evaluate."""
        return command

    def evaluate(self, code):
        """Evaluate code; return the result of evaluating code in the object system for this lingo.
        code should be an object returned by read."""
        pass

    def string(self, obj):
        """Convert obj to a string.  obj is an object in the object system for this lingo."""
        return str(obj)

    def python(self, obj):
        """Convert obj to the appropriate python type.  obj is an object in the object system for
        this lingo."""
        return obj
  
    def execute(self, command):
        """Equivalent to self.python(self.evaluate(self.read(command)))."""
        return self.python(self.evaluate(self.read(command)))
