#!/usr/bin/env python
# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/lisp/lisp.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

# A simple lisp interpreter written in Python.

# To Do:
# Add support for:
# defun
# defmacro
# let
# do

# Optimize the code:
# The current implementation is simple but very inefficient (especially environements, this can be fixed by using a
# dictionary of lists instead of a list of dictionaries).

import sys

if __name__ == '__main__':
    sys.path.append('../../../../')

from boots.lib.lingos import lingo
from boots.lib.lingos.lisp import builtins
from boots.lib.lingos.lisp import lexer
from boots.lib.lingos.lisp import objects
from boots.lib.lingos.lisp import parser
import re
import readline

def input_complete(string):
    """Returns string if string is a complete lisp expression.  Otherwise, None is returned."""
    # Remove strings as they may contain parentheses.
    tmp = re.sub(lexer.t_STRING, '', string)
    left_parens = tmp.count('(')
    right_parens = tmp.count(')')
    paren_level = left_parens - right_parens

    if paren_level < 0:
        raise SyntaxError('unmatched closing ")"')
    elif paren_level > 0:
        return None
    else:
        return string

def read_string(string):
    """Reads string and returns a lisp object."""
    return parser.parser.parse(string)

def read(stream = sys.stdin):
    """Reads a lisp object from stream.  stream may be either a file or a callable in which case each call to stream should
    return the next line."""
    paren_level = 0
    lines = ''
    
    if isinstance(stream, file):
        stream = stream.readline
    elif not callable(stream):
        raise TypeError('{0} is not a stream or a callable'.format(stream))
    # Read lines until all open lisp expressions have been closed.
    while True:
        line = stream()
        # Put a space between arguments separated by a newline to prevent concatenation.
        lines += ' ' + line

        if input_complete(lines):
            break
        
    return read_string(lines)

def evaluate(expression, environment):
    """Evaluates expression in environment and returns the result."""
    return expression.evaluate(environment)

class LispInterpreter(lingo.Interpreter):
    def __init__(self, *arguments, **keywords):
        super(LispInterpreter, self).__init__(*arguments, **keywords)
        self.global_environment = objects.Environment()
        builtins.initialize(self.global_environment, self.stdin, self.stdout)

    def input_complete(self, command):
        return input_complete(command)
    
    def read(self, command):
        if command.strip():
            return read_string(command)

    def evaluate(self, code):
        if code != None:
            return evaluate(code, self.global_environment)

lingo.register('lisp', LispInterpreter)

def main():
    try:
        global_environment = objects.Environment()
        builtins.initialize(global_environment)
        
        while True:
            print(evaluate(read(lambda: raw_input('> ')), global_environment))
            
    except EOFError:
        pass

if __name__ == '__main__':
    main()
