# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/lisp/parser.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####
# Defines the lisp parser.

from lexer import tokens
from boots.lib.lingos.lisp import builtins
from boots.lib.lingos.lisp import objects
import ply.yacc as yacc

def p_expression_integer(p):
    'expression : INTEGER'
    p[0] = objects.Integer(int(p[1]))

def p_expression_string(p):
    'expression : STRING'
    p[0] = objects.String(p[1][1:len(p[1]) - 1])

def p_expression_symbol(p):
    'expression : SYMBOL'
    p[0] = objects.Symbol(p[1])

def p_expression_quoted(p):
    'expression : quoted_expression'
    p[0] = p[1]

def p_expression(p):
    'expression : LEFT_PAREN expression_list RIGHT_PAREN'
    p[0] = p[2]

def p_quoted_expression(p):
    'quoted_expression : QUOTE expression'
    p[0] = objects.Quote(p[2])

def p_expression_list_empty(p):
    'expression_list : '
    p[0] = builtins.nil

def p_expression_list(p):
    'expression_list : expression expression_list'
    p[0] = objects.Cons(p[1], p[2])

def p_error(p):
    print('syntax error')

# Disable debugging output and caching of parse tables.
parser = yacc.yacc(debug = 0, write_tables = 0)
