# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/python.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

from boots.lib.lingos import lingo

class PythonInterpreter(lingo.Interpreter):
    """Implements a python interpreter that will send commands to the python interpreter that runs
    boots."""
    def __init__(self, *args, **kwargs):
        super(PythonInterpreter, self).__init__(*args, **kwargs)
        # These dictionaries are used to store the variables.
        self._globals = {"console" : self.console}
        self._locals = {}
    
    def input_complete(self, command):
        # Consider input complete if the last line did not end with ":", the last two characters
        # were newlines or it was empty.
        try:
            if command[-2] != ':' or command[-2:] == '\n\n':
                return command
            else:
                return None
        except IndexError:
            return command

    def evaluate(self, code):
        if code.strip():
            try:
                return eval(code, self._globals, self._locals)
            # If eval results in an error, it might be because code is a statement rather than an expression.
            except:
                try:
                    exec code in self._globals, self._locals
                except BaseException as e:
                    print(e)
                    
lingo.register('python', PythonInterpreter)
