# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/sql.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

import itertools
import sys
from boots.lib.lingos import lingo

class SQLInterpreter(lingo.Interpreter):
    def __init__(self, console, stdin = sys.stdin, stdout = sys.stdout):
        super(SQLInterpreter, self).__init__(console, stdin, stdout)
        self.terminating_char = console.config["terminating_char"]

    def input_complete(self, command):
        try:
            # FIXME ignore leading whitespace when testing prefixes.
            # Comments should be removed when in the middle of a statement
            # as well.
            if command.startswith("\\") or command.startswith("-- ") or \
               self.terminating_char == "\n":
                return command
            # FIXME ignore trailing whitespace when testing terminators.
            elif command.strip("\n").endswith(self.terminating_char):
                return command[0:-len(self.terminating_char)]
            else:
                return None
        except IndexError:
            return command

    def evaluate(self, code):
        if code.strip():
            return itertools.chain(*(server.execute(code) for server in self.console.servers))

lingo.register('sql', SQLInterpreter)