# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/console.py 
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

import warnings

def parse_metacommand(commandstring):
    """This function wraps str.split() for metacommands

    This function takes a string as argument and calls str.split() on that string
    to return a list of substrings, using whitespace as separator. The metacommand
    object uses this list to call a metacommand function and pass parameters.
    
    """ 
    return commandstring.split()    

class MetaCommandManager(object):
    def __init__(self):
        self.names = set()
        
    def add(self, metacommands):
        newnames = set(metacommands.commands.iterkeys())
        if not self.names.isdisjoint(newnames):
            # FIXME: Issue warning through the UI using ui.print
            # FIXME: Determine the offending command(s) (set.intersect?)
            # FIXME: Print offending commands
            warnings.warn("Conflicts between metacommands detected.")
        self.names |= newnames

class MetaCommands(object):
    """Class used for handling metacommands.

    The MetaCommands class provides a mapping of command strings to function
    references. Its functions are the constructor, parse() (a function not bound
    to a MetaCommands object) and execute().
    
    The constructor accepts one parameter, a dictionary of string to function
    reference mappings. It stores this parameter in the member variable commands.
    
    """
    commands = {}

    def __init__(self, commanddict):
        self.commands = commanddict

    def execute(self, commandlist):
        """Parse and execute a metacommand string.

        The execute() function takes a list of strings, with the first being the name
        of the metacommand to be executed and the others being the required parameters.
        It performs a dictionary lookup on the name and calls the function it finds,
        or returns False if no function is found.
        
        """
        if len(commandlist) != 0 and commandlist[0] in self.commands:
            self.commands[commandlist[0]](*commandlist[1:])
            #Do we need to pass metacommand return values back?
            return True
        else:
            return False
