# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# boots/lib/ui/generic.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

import sys
import StringIO

class StreamDriver(object):
    def __init__(self, console, stream, lingo):
        self.console = console
        self.stream = stream
        self.lingo = lingo

    def get_input(self):
        """Gets user input from a stream."""
        try:
            lines = ""
            for line in self.stream:
                lines += line
                command = self.console.input_complete(lines, self.lingo)
                if command:
                    yield command, self.lingo
                    lines = ""
                                
        except EOFError:
            return
        
    def unload(self):
        pass

class StreamPresenter(object):
    def __init__(self, stream):
        self.stream = stream
        
    def present(self, row):
        """Prints row information in a "raw" format"""
        self.stream.write(str(row))
        self.stream.write("\n")
        
class StdinDriver(StreamDriver):
    def __init__(self, console, lingo):
        super(StdinDriver, self).__init__(sys.stdin, console, lingo)

class StdoutPresenter(StreamPresenter):
    def __init__(self,):
        super(StdoutPresenter, self).__init__(sys.stdout)
        
class ScriptDriver(StreamDriver):
    def __init__(self, console, path, lingo):
        self.path = path
        self.script_file = open(path)
        super(ScriptDriver, self).__init__(console, self.script_file, lingo)
        
class StringDriver(StreamDriver):
    def __init__(self, console, input_string, lingo):
        self.input_string = input_string
        string_stream = StringIO.StringIO(self.input_string)
        super(StringDriver, self).__init__(console, string_stream, lingo)
