# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/ui/plain.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

import readline
import sys

from boots.lib.ui.components.metacommands import MetaCommands, parse_metacommand
from boots.lib.ui.generic import StdinDriver, StdoutPresenter

class PlainUI(StdinDriver, StdoutPresenter):
    def __init__(self, console, prompt1, prompt2, hist_length, hist_file, lingo):
        self.metacommands = MetaCommands({
            '\lingo': self.switchlingo})
        self.console = console
        self.prompt1 = prompt1
        self.prompt2 = prompt2
        self.hist_file = hist_file
        self.lingo = lingo
        self.current_widths = None
        self.buffer = []
        
        # FIXME: This should go in an initialization function
        readline.set_history_length(hist_length)
        self.read_history()

    def read_history(self):
        try:
            readline.read_history_file(self.hist_file)
        except IOError:
            pass

    def save_history(self):
        readline.write_history_file(self.hist_file)
        
    def unload(self):
        self.save_history()

    def get_input(self):
        """Gets user input from stdin via a prompt."""
        while True:
            try:
                line = raw_input(self.prompt1)
                
                if not self.metacommands.execute(parse_metacommand(line)):
                    command = self.console.input_complete(line, self.lingo)
                    lines = line + "\n"
                    while command is None:
                        lines += raw_input(self.prompt2) + "\n"
                        command = self.console.input_complete(lines, self.lingo)
                                        
                    yield command, self.lingo
                
            except EOFError:
                return
            
    def set_prompt(self, prompt):
        self.prompt = prompt

    def present(self, result):
        def get_dashes(x):
            return "-" * (x+2)
        try:
            if not self.current_widths:
                self.query = result["__server_execute_sql_query"]
                self.names = [column[0] for column in result["info"]]
                self.current_widths = map(len, self.names)
                self.buffer = []
            for row in result["result"]:
                new_row = map(str, row)
                widths = map(len, new_row)
                self.current_widths = map(max, widths, self.current_widths)
                self.buffer.append(new_row)
        except (TypeError, KeyError, StopIteration):
            if self.current_widths:
                dashes = map(get_dashes, self.current_widths)
                sep = "+" + "+".join(dashes) + "+\n"
                names = map("{0:{1}}".format, self.names, self.current_widths)
                column_names = "| " + " | ".join(names) + " |\n"
                sys.stdout.write(sep)
                sys.stdout.write(column_names)
                sys.stdout.write(sep)
                for row in self.buffer:
                    strs = "| " + " | ".join(map("{0:{1}}".format, row, self.current_widths)) + " |\n"
                    sys.stdout.write(strs)
                sys.stdout.write(sep)
                # Reset values for next result set.
                self.current_widths = None
            print(result)
        except:
            # Reset values for next result set.
            self.current_widths = None

    #FIXME: Is there / can there be a list of available lingos?
    #If so, we could implement dynamic registration of lingos through this
    #interface: Register the lingo used in main(), and register lingos
    #switched to here.
    def switchlingo(self, lingo):
        self.lingo = lingo
