import unittest
from optparse import OptionParser
import sys
import os
import os.path
from boots_unit_test import BootsBaseTester
# Path mangling to import boots files properly.
new_path = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))),
                        "src")
sys.path.insert(0, new_path)
from api import api_tests
from api.nodes import node_tests
from app import app_tests
from lib import console_tests
from lib.lingos import lingo_tests
from lib.ui import ui_tests

if __name__ == "__main__":
    parser = OptionParser()
    parser.add_option("-H", "--host", dest="host", default=BootsBaseTester.config["host"],
                      help="Host running the Drizzle server on which the test\
                            DB should be run.")
    parser.add_option("-p", "--port", type="int", dest="port", default=BootsBaseTester.config["port"],
                      help="Port the Drizzle server is running on.")
    parser.add_option("-D", "--database", dest="database", default=BootsBaseTester.config["database"],
                      help="Database to use for testing on the Drizzle server.")
    (options, args) = parser.parse_args()
    BootsBaseTester.config = vars(options)
    if not args:
        modules = [api_tests, node_tests, app_tests, console_tests, lingo_tests, ui_tests]
    else:
        modules = map(globals().__getitem__, args)
    for module in modules:
        suite = unittest.TestLoader().loadTestsFromModule(module)
        unittest.TextTestRunner(verbosity=2).run(suite)
